<?php
/* --------------------------------------------------------------
   GoogleAnalyticsProxyController.inc.php 2018-06-18
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2018 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

class GoogleAnalyticsProxyController extends HttpViewController
{
	/**
	 * Proxies the google analytics request through the shop system.
	 */
	public function actionTunnelRequest()
	{
		session_write_close();
		$query = $this->_getQueryParametersCollection()->getArray();
		
		// remove controller and action from do query param
		$doQueryParams = explode('/', $query['do']);
		array_shift($doQueryParams);
		array_shift($doQueryParams);
		
		// create the path which gets append to the www.google-analytics.com domain
		$gPath = '';
		foreach($doQueryParams as $doQueryParam)
		{
			$gPath .= '/' . $doQueryParam;
		}
		
		// parses the google path to extract the version query param
		$parsedGPath = parse_url($gPath);
		if(array_key_exists('query', $parsedGPath))
		{
			$querySegments = explode('=', $parsedGPath['query']);
			$query         = array_merge([$querySegments[0] => $querySegments[1]], $query);
		}
		unset($query['do']);
		
		// creates the final google analytics url
		$gUrl = 'https://www.google-analytics.com' . $parsedGPath['path'];
		
		// try to fetch the client ip
		$clientIp = $this->_getClientIp();
		if($clientIp)
		{
			$query['uip'] = $clientIp;
		}
		if(array_key_exists('HTTP_USER_AGENT', $_SERVER))
		{
			$query['ua'] = $_SERVER['HTTP_USER_AGENT'];
		}
		
		// sends the analytics data to the google servers
		$finalUrl  = $gUrl . '?' . http_build_query($query);
		$gCurl     = curl_init($finalUrl);
		$gResponse = curl_exec($gCurl);
		curl_close($gCurl);
		
		header('Content-Type: image/gif');
		echo $gResponse;
		die;
	}
	
	
	private function _getClientIp()
	{
		$remoteAddressKey = 'REMOTE_ADDR';
		$proxyHeader      = 'HTTP_X_FORWARDED_FOR';
		
		if(!array_key_exists($remoteAddressKey, $_SERVER))
		{
			return null;
		}
		
		if(array_key_exists($proxyHeader, $_SERVER))
		{
			$proxies = explode(',', $_SERVER[$proxyHeader]);
			
			$clientIp = $proxies[0];
			
			if(filter_var($clientIp, FILTER_VALIDATE_IP))
			{
				return $clientIp;
			}
		}
		
		return $_SERVER[$remoteAddressKey];
	}
}